@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2015 Tada AB and other contributors, as listed below.
+ * Copyright (c) 2004-2019 Tada AB and other contributors, as listed below.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the The BSD 3-Clause License
@@ -20,7 +20,12 @@ import java.sql.Savepoint;
  * savepoints. To receive such notifications, implement this interface, with
  * the three methods that will be called in those three cases, and pass an
  * instance to {@link Session#addSavepointListener}.
- *
+ *<p>
+ * It is possible for a listener method to be called with <em>savepoint</em>
+ * null, or <em>parent</em> null, or both; that can happen if the application
+ * code has not kept a strong reference to the {@code Savepoint} object in
+ * question.
+ *<p>
  * <code>SavepointListener</code> exposes a
  * <a href=
 'http://doxygen.postgresql.org/xact_8h.html#aceb46988cbad720cc8a2d7ac6951f0ef'
@@ -30,9 +35,12 @@ import java.sql.Savepoint;
  */
 public interface SavepointListener
 {
-	void onAbort(Session session, Savepoint savepoint, Savepoint parent) throws SQLException;
+	void onAbort(Session session, Savepoint savepoint, Savepoint parent)
+	throws SQLException;
 
-	void onCommit(Session session, Savepoint savepoint, Savepoint parent) throws SQLException;
+	void onCommit(Session session, Savepoint savepoint, Savepoint parent)
+	throws SQLException;
 
-	void onStart(Session session, Savepoint savepoint, Savepoint parent) throws SQLException;
+	void onStart(Session session, Savepoint savepoint, Savepoint parent)
+	throws SQLException;
 }
