@@ -800,6 +800,9 @@ Datum Function_invoke(Function self, PG_FUNCTION_ARGS)
 	if(self->isUDT)
 		return self->func.udt.udtFunction(self->func.udt.udt, fcinfo);
 
+	/* a class loader or other mechanism might have connected already. This
+	 * connection must be dropped since its parent context is wrong.
+	 */
 	if(self->func.nonudt.isMultiCall && SRF_IS_FIRSTCALL())
 		Invocation_assertDisconnect();
 
@@ -816,9 +819,6 @@ Datum Function_invoke(Function self, PG_FUNCTION_ARGS)
 		int32 idx;
 		Type* types = self->func.nonudt.paramTypes;
 
-		/* a class loader or other mechanism might have connected already. This
-		 * connection must be dropped since its parent context is wrong.
-		 */
 		if(Type_isDynamic(invokerType))
 			invokerType = Type_getRealType(invokerType, get_fn_expr_rettype(fcinfo->flinfo), self->func.nonudt.typeMap);
 
@@ -853,7 +853,7 @@ Datum Function_invokeTrigger(Function self, PG_FUNCTION_ARGS)
 	Datum  ret;
 
 	TriggerData *td = (TriggerData*)fcinfo->context;
-	arg.l = TriggerData_create(td);
+	arg.l = pljava_TriggerData_create(td);
 	if(arg.l == 0)
 		return 0;
 
@@ -888,7 +888,8 @@ Datum Function_invokeTrigger(Function self, PG_FUNCTION_ARGS)
 #endif
 		MemoryContext currCtx = Invocation_switchToUpperContext();
 		ret = PointerGetDatum(
-				TriggerData_getTriggerReturnTuple(arg.l, &fcinfo->isnull));
+				pljava_TriggerData_getTriggerReturnTuple(
+					arg.l, &fcinfo->isnull));
 
 		/* Triggers are not allowed to set the fcinfo->isnull, even when
 		 * they return null.
